@@ -187,7 +187,7 @@ class SQLExpression:
     def tablesUsedDict(self):
         tables = {}
         for table in self.tablesUsedImmediate():
-            tables[table] = 1
+            tables[str(table)] = 1
         for component in self.components():
             tables.update(tablesUsedDict(component))
         return tables
@@ -505,11 +505,17 @@ class Select(SQLExpression):
         if self.whereClause is not NoDefault:
             select += " WHERE %s" % sqlrepr(self.whereClause, db)
         if self.groupBy is not NoDefault:
-            select += " GROUP BY %s" % sqlrepr(self.groupBy, db)
+            groupBy = sqlrepr(self.groupBy, db)
+            if isinstance(self.groupBy, list) or isinstance(self.groupBy, tuple):
+                groupBy = groupBy[1:-1] # Remove parens
+            select += " GROUP BY %s" % groupBy
         if self.having is not NoDefault:
             select += " HAVING %s" % sqlrepr(self.having, db)
         if self.orderBy is not NoDefault:
-            select += " ORDER BY %s" % sqlrepr(self.orderBy, db)
+            orderBy = sqlrepr(self.orderBy, db)
+            if isinstance(self.orderBy, list) or isinstance(self.orderBy, tuple):
+                orderBy = orderBy[1:-1] # Remove parens
+            select += " ORDER BY %s" % orderBy
         if self.limit is not NoDefault:
             select += " LIMIT %s" % sqlrepr(self.limit, db)
         return select
@@ -602,10 +608,12 @@ class Delete(SQLExpression):
             raise TypeError, "You must give a where clause or pass in None to indicate no where clause"
         self.whereClause = where
     def __sqlrepr__(self, db):
-        if self.whereClause is None:
+        whereClause = self.whereClause
+        if whereClause is None:
             return "DELETE FROM %s" % self.table
-        return "DELETE FROM %s WHERE %s" \
-               % (self.table, sqlrepr(self.whereClause, db))
+        if isinstance(whereClause, SQLExpression):
+            whereClause = sqlrepr(whereClause, db)
+        return "DELETE FROM %s WHERE %s" % (self.table, whereClause)
 
 registerConverter(Delete, SQLExprConverter)
 
@@ -667,14 +675,14 @@ def NOTIN(item, list):
     else:
         return NOT(_IN(item, list))
 
-def STARTSWITH(expr, string):
-    return SQLOp("LIKE", expr, _LikeQuoted(string) + '%')
+def STARTSWITH(expr, pattern):
+    return SQLOp("LIKE", expr, _LikeQuoted(pattern) + '%')
 
-def ENDSWITH(expr, string):
-    return SQLOp("LIKE", expr, '%' + _LikeQuoted(string))
+def ENDSWITH(expr, pattern):
+    return SQLOp("LIKE", expr, '%' + _LikeQuoted(pattern))
 
-def CONTAINSSTRING(expr, string):
-    return SQLOp("LIKE", expr, '%' + _LikeQuoted(string) + '%')
+def CONTAINSSTRING(expr, pattern):
+    return SQLOp("LIKE", expr, '%' + _LikeQuoted(pattern) + '%')
 
 def ISNULL(expr):
     return SQLOp("IS", expr, None)
@@ -683,6 +691,8 @@ def ISNOTNULL(expr):
     return SQLOp("IS NOT", expr, None)
 
 class _LikeQuoted:
+    # It assumes prefix and postfix are strings; usually just a percent sign.
+
     # @@: I'm not sure what the quoting rules really are for all the
     # databases
 
@@ -700,12 +710,29 @@ class _LikeQuoted:
         return self
 
     def __sqlrepr__(self, db):
-        s = sqlrepr(self.expr, db)[1:-1] # remove quotes
-        if db in ('postgres', 'mysql'):
-            s = s.replace('%', '\\%')
-        else:
-            s = s.replace('%', '%%')
-        return "'%s%s%s'" % (self.prefix, s, self.postfix)
+        s = self.expr
+        if isinstance(s, SQLExpression):
+            values = []
+            if self.prefix:
+                values.append("'%s'" % self.prefix)
+            s = _quote_percent(sqlrepr(s, db), db)
+            values.append(s)
+            if self.postfix:
+                values.append("'%s'" % self.postfix)
+            if db == "mysql":
+                return "CONCAT(%s)" % ", ".join(values)
+            else:
+                return " || ".join(values)
+        else: # assuming s is a string
+            s = _quote_percent(s, db)
+            return "'%s%s%s'" % (self.prefix, s, self.postfix)
+
+def _quote_percent(s, db):
+    if db in ('postgres', 'mysql'):
+        s = s.replace('%', '\\%')
+    else:
+        s = s.replace('%', '%%')
+    return s
 
 ########################################
 ## SQL JOINs
@@ -715,12 +742,12 @@ class SQLJoin(SQLExpression):
     def __init__(self, table1, table2, op=','):
         if table1 and type(table1) <> str:
             if isinstance(table1, Alias):
-                table1 = "%s AS %s" % (table1.q.tableName, table1.q.alias)
+                table1 = "%s %s %s" % (table1.q.tableName, AliasField.as_string, table1.q.alias)
             else:
                 table1 = table1.sqlmeta.table
         if type(table2) <> str:
             if isinstance(table2, Alias):
-                table2 = "%s AS %s" % (table2.q.tableName, table2.q.alias)
+                table2 = "%s %s %s" % (table2.q.tableName, AliasField.as_string, table2.q.alias)
             else:
                 table2 = table2.sqlmeta.table
         self.table1 = table1
